<?php
/**
 *.Hack Name:        Shop Hack
 * Version:          3.4
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

if (!defined('IN_CODE')) {
    header('HTTP/1.0 403 Forbidden');
    exit("Not allowed to run this file directly.");
}

/**
 * Add a record to the Bank
 *
 * @param int $fromuid
 * @param int $touid
 * @param int $amount
 * @param string $comment Must be HTML safe and SQL safe.
 * @param string $type Transaction type code, must be SQL safe.
 */
function AddBankRecord($fromuid, $touid, $amount, $comment, $type) {
    global $db;

    $fromuid = (int)$fromuid;
    $touid = (int)$touid;
    $amount = (int)$amount;

    $db->query(
        "INSERT INTO ".X_PREFIX."shop_bank "
      . "(fromuid,  touid,  dateline,   amount,  comment,  type) VALUES "
      . "('$fromuid', '$touid', '".time()."', '$amount', '$comment', '$type')"
    );
}

/**
 * A reversed nl2br-function
 */
function br2nl($str) {
    $str = preg_replace("/(\r\n|\n|\r)/", "", $str);
    return preg_replace("=<br */?>=i", "\n", $str);
}

/**
 * checkOnOff()
 *
 * Check var for either ON or OFF value
 *
 * @param    $str        The string
 * @param    $default    Default value 'on' or 'off'
 * @return   The value on/off
 */
function checkOnOff($str, $default='off') {
    // expects a string
    if ( safeString($str, false) === 'on' ) {
        return 'on';
    } else {
        return 'off';
    }
}

/**
 * formatCash()
 *
 * Check var for either ON or OFF value
 *
 * @param    $amount     The amount of money
 * @return   A formatted value
 */
function formatCash($amount='0') {
    return number_format($amount);
}

/**
 * privfcheck()
 *
 * Check if someone has access or not
 *
 * @param    $private
 * @param    $userlist   UserAccessList
 * @return   true = access, false = no access
 */
function privfcheck($private, $userlist) {
    global $self, $xmbuser;

    // Super admins can see all forums
    if ( X_SADMIN ) {
        return true;
    }

    switch($private) {
        case 4: // 4 - No Viewing
            return false;
        break;

        case 3: // 3 - Mods/Admins
            return X_STAFF;
        break;

        case 2: // 2 - Admins
            return X_ADMIN;
        break;

        case 1: // 1 - All users... minus guests.
            if (trim($userlist) == '') {
                return true;
            }

            $user = explode(',', $userlist);
            $xuser = strtolower($xmbuser);

            foreach ($user as $usr) {
                $usr = strtolower(trim($usr));
                if ( $usr != '' && $xuser == $usr ) {
                   return true;
                }
            }
            return false;
        break;

        default:
            return false;
        break;
    }
    return false;
}

/**
 * ShopAccess()
 *
 * Check to see if someone has access to the Shop
 */
function ShopAccess($do = '0') {
    global $self, $SHOP;

    $string = ($do == 0 ? $SHOP['AccessShop'] : $SHOP['AccessCats']);

    $T = explode("|FF|", $string);
    $T['2'] = strtolower($T['2']);
    $T['3'] = strtolower($T['3']);
    $U = strtolower($self['username']);

    if ( X_GUEST ) { // Guests
        $ret = false;
    } else if ( X_SADMIN ) { // Super Admins
        $ret = true;
    } else if (  ($T['0'] > 0) && ($self['postnum'] < $T['0']) ) { // Posts
        $ret = false;
    } else if ( false !== strpos($T['3'], '|du|'.$U.'|du|') ) { // Denied Username
        $ret = false;
    } else if ( false !== strpos($T['2'], '|au|'.$U.'|au|') ) { // Allowed Username
        $ret = true;
    } else if ( false !== strpos($T['1'], '|DS|'.$self['status'].'|DS|') ) { // Denied Status
        $ret = false;
    } else if ( false !== strpos($T['1'], '|AS|'.$self['status'].'|AS|') ) { // Allowed Status
        $ret = true;
    } else { // Anything else ?
        $ret = false;
    }

    return $ret;
}

/**
 * ShopLinks()
 *
 * Creates the template with links
 *
 * Roxas:    Since I'm to lazy to write a function
 * FunForum: Fine, then I'll do it ...
 */
function ShopLinks($cat=NULL, $iid='0') {
    global $lang, $self, $SHOP, $THEME;

    $newitemlink = '';
    $auth = ShopAccess('1');

    $dlink1 = $dlink2 = '';

    $dlink1 = '<a href="shop_bank.php">'.$lang['shop_textbank'].'</a> ';

    $dlink1 .= '| <a href="shop_info.php?action=moneyinfo" onclick="Popup(this.href, \'shopWindow\', 400, 200); return false;">'.$lang['shop_moneyinfo'].'</a> ';

    if ( $SHOP['search'] == "on" ) {
        $dlink1 .= '| <a href="shop_info.php?action=search">'.$lang['shop_textsearch'].'</a> ';
    }

    if ( $SHOP['stats'] == "on" ) {
        $dlink1 .= '| <a href="shop_info.php?action=stats">'.$lang['shop_textstats'].'</a> ';
    }

    if ( X_ADMIN ) {
        $dlink1 .= '| <a href="shop_cp.php">'.$lang['shop_textadminpanel'].'</a> ';
    }

    if ( false !== $auth ) {
        $dlink1 .= '| <a href="shop_edit.php?action=newcat">'.$lang['shop_textnewcat'].'</a> ';
    }

    // Category links: shop.php?cid=     and     shop_item.php
    if (is_array($cat)) {
        if(($cat['creatoronly'] == "yes" && $cat['owner'] == $self['uid']) || false !== $auth) {
            $dlink2 .= '| <a href="shop_edit.php?action=editcat&amp;cid='.$cat['id'].'">'.$lang['shop_texteditcat'].'</a> ';
            $dlink2 .= '| <a href="shop_edit.php?action=deletecat&amp;cid='.$cat['id'].'">'.$lang['shop_textdeletecat'].'</a> ';
            $newitemlink = '| <a href="shop_edit.php?action=newitem&amp;cid='.$cat['id'].'">'.$lang['shop_textnewitem'].'</a> ';
        }

        $private = explode("|", $cat['private']);
        $userlist = explode("|", $cat['userlist']);
        $authorization = privfcheck($private[0], $userlist[0]);
        if(($cat['creatoronly'] == "yes" && $cat['owner'] == $self['uid']) || false !== $authorization) {
            if($SHOP['subcats'] == "on" && $cat['cid'] == 0) {
                $dlink2 .= '| <a href="shop_edit.php?action=newsubcat&amp;cid='.$cat['id'].'">'.$lang['shop_textnewsubcat'].'</a> ';
            }
            $newitemlink = '| <a href="shop_edit.php?action=newitem&amp;cid='.$cat['id'].'">'.$lang['shop_textnewitem'].'</a> ';

            if ( $iid > 0 ) { // shop_item.php
                $dlink2 .= '| <a href="shop_edit.php?action=edititem&amp;iid='.$iid.'">'.$lang['shop_textedititem'].'</a> ';
                $dlink2 .= '| <a href="shop_edit.php?action=deleteitem&amp;iid='.$iid.'">'.$lang['shop_textdeleteitem'].'</a> ';
            }
        }
    }

    $dlink2 .= $newitemlink;

    $dlink1 .= (empty($dlink2) ? '' : '<br />'.substr($dlink2, 1));

    $output = '<table cellspacing="0" cellpadding="0" border="0" width="'.$THEME['tablewidth'].'" align="center">
    <tr><td bgcolor="'.$THEME['bordercolor'].'">
    <table border="0" cellspacing="'.$THEME['borderwidth'].'" cellpadding="'.$THEME['tablespace'].'" width="100%">
    <tr><td class="tablerow" bgcolor="'.$THEME['altbg2'].'">'.$dlink1.'</td></tr>
    </table>
    </td></tr></table>
    <br />';

    return $output;
}

/**
 * recalc_lasts()
 *
 * This will recount the 'last added' of a category.
 * In the total, the items located in sub-categories will count as well.
 * Sub-categories will also be recounted.
 */
function recalc_lasts($CID) {
    global $db; //, $table_shop_cats, $table_shop_items, $table_members;

    $CID = (int)$CID;
    $arr = array(); // collect the newest items (in the sub-categories) in an array

    // Get the sub-categories info
    $queryA = $db->query("SELECT id, cid FROM ".X_PREFIX."shop_cats WHERE cid=".$CID);
    while ($SubCat = $db->fetch_array($queryA)) {

        // Search for latest item in that category
        $queryB = $db->query("SELECT m.username, s.id, s.owner, s.dateline FROM ".X_PREFIX."shop_items s, ".X_PREFIX."members m WHERE (s.cid=".$SubCat['id']." AND m.uid=s.owner) ORDER BY s.dateline DESC limit 0, 1");
        $cat_item = $db->fetch_array($queryB);
        if ( !empty($cat_item['id']) ) {
            $last = $cat_item['dateline']."|".$cat_item['username'];
            $arr[] = $last;
        } else {
            $last = '';
        }
        // Update category
        $db->query("UPDATE ".X_PREFIX."shop_cats SET lastadd='$last' WHERE id=".$SubCat['id']);
    }

    // check the category itself
    $queryX  = $db->query("SELECT m.username, s.id, s.owner, s.dateline FROM ".X_PREFIX."shop_items s, ".X_PREFIX."members m WHERE (s.cid=".$CID." AND m.uid=s.owner) ORDER BY s.dateline DESC limit 0, 1");
    $mc_item = $db->fetch_array($queryX);
    if ( !empty($mc_item['id']) ) {
        $arr[] = $mc_item['dateline']."|".$mc_item['username'];
    }

    // Update category
    rsort($arr);
    $last = (empty($arr) ? '' : $arr['0']);
    $db->query("UPDATE ".X_PREFIX."shop_cats SET lastadd='$last' WHERE id=".$CID);
}

/**
 * recalc_itemcount()
 *
 * This will recount the amount of items that are in a category.
 * In the total, the items located in sub-categories will count as well.
 * Sub-categories will also be recounted.
 */
function recalc_itemcount($CID) {
    global $db; //, $table_shop_cats, $table_shop_items;

    $CID = (int)$CID;
    $itemnum     = 0;
    $sub_itemnum = 0;
    $squery  = '';
    $stquery = '';
    $iquery  = '';

    // Get all items for the category's sub-categories
    $squery = $db->query("SELECT id FROM ".X_PREFIX."shop_cats WHERE cid=".$CID);
    while ($sub = $db->fetch_array($squery)) {

        $stquery = $db->query("SELECT COUNT(id) FROM ".X_PREFIX."shop_items WHERE cid=".$sub['id']);
        $sub_itemnum = $db->result($stquery, 0);

        $db->query("UPDATE ".X_PREFIX."shop_cats SET items=$sub_itemnum WHERE id=".$sub['id']);
        $itemnum += $sub_itemnum;
    }
    $db->free_result($squery);

    // Get all items for the category itself
    $iquery = $db->query("SELECT COUNT(id) FROM ".X_PREFIX."shop_items WHERE cid=".$CID);
    $itemnum += $db->result($iquery, 0);

    // Update it all
    $db->query("UPDATE ".X_PREFIX."shop_cats SET items='$itemnum' WHERE id=".$CID);
}

?>
